@@ -38,7 +38,10 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     while let Some(container) = reader.read_data_container()? {
         for slice in container.slices() {
-            for record in slice.records(container.compression_header())? {
+            let records = slice.records(container.compression_header())?;
+            let records = slice.resolve_mates(records)?;
+
+            for record in records {
                 let sam_record = record.try_into_sam_record(
                     &reference_assembly,
                     header.reference_sequences(),
